from errno import EBADF
import os

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from StringIO import _complain_ifclosed

from django.conf import settings
from django.db import models

from django_dbstorage.fields import Base64Field


__all__ = ['File']

class FileManager(models.Manager):
    def open(self, name, mode):
        f = self.get(name=name)
        f.mode = mode
        return f


class File(models.Model):
    """Model for treating rows in the database as file streams."""

    name = models.CharField(max_length=getattr(settings, 'PATH_MAX', 100),
                            primary_key=True, editable=False)
    data = Base64Field(blank=True, editable=False)
    size = models.PositiveIntegerField(editable=False)

    objects = FileManager()

    def __init__(self, *args, **kwargs):
        mode = kwargs.pop('mode', 'r+b')
        super(File, self).__init__(*args, **kwargs)
        self._closed = False
        self._file = StringIO()
        self._file.write(self.data)
        self._file.seek(0)
        self.encoding = None
        self.errors = None
        self.mode = mode
        self.newlines = None
        self.softspace = 0

    def __unicode__(self):
        output = '%r, mode %r' % (self.name, self.mode)
        if self.closed:
            output = 'closed ' + output
        return output

    def __iter__(self):
        return self

    def next(self):
        _complain_ifclosed(self.closed)
        r = self._file.readline()
        if not r:
            raise StopIteration
        return r

    def open(self, mode=None):
        """open([mode]) -> None. """
        if self._file is None:
            raise ValueError("The file cannot be reopened.")
        self._closed = False
        self._file.seek(0)
        if mode:
            self.mode = mode

    def close(self):
        """close() -> None.  Flushes and releases resources held."""
        if not self.closed:
            if not self._isreadonly():
                self.flush()
            self._closed = True

    def _get_closed(self):
        return self._closed or self._file is None
    closed = property(_get_closed)

    def isatty(self):
        """isatty() -> False."""
        _complain_ifclosed(self.closed)
        return False

    def seek(self, offset, whence=0):
        """
        seek(offset[, whence]) -> None.  Move to new file position.

        Argument offset is a byte count.  Optional argument whence defaults to
        0 (offset from start of file, offset should be >= 0); other values are 1
        (move relative to current position, positive or negative), and 2 (move
        relative to end of file, usually negative, but allows seeking beyond
        the end of a file).
        """
        _complain_ifclosed(self.closed)
        return self._file.seek(offset, whence)

    def tell(self):
        """
        tell() -> current file position, an integer (may be a long integer).
        """
        _complain_ifclosed(self.closed)
        return self._file.tell()

    def read(self, size=-1):
        """read([size]) -> read at most size bytes, returned as a string."""
        _complain_ifclosed(self.closed)
        return self._file.read(size)

    def readline(self):
        """readline() -> next line from the file, as a string."""
        _complain_ifclosed(self.closed)
        return self._file.readline()

    def readlines(self):
        """readlines() -> list of strings, each a line from the file."""
        _complain_ifclosed(self.closed)
        return self._file.readlines()

    def truncate(self, size=None):
        """
        truncate([size]) -> None.  Truncate the file to at most size bytes.

        Size defaults to the current file position, as returned by tell().
        """
        _complain_ifclosed(self.closed)
        _complain_ifreadonly(self._isreadonly())
        position = self._file.tell()
        try:
            if size is not None:
                self._file.seek(size)
            return self._file.truncate()
        finally:
            self._file.seek(position)

    def write(self, str):
        """
        write(str) -> None.  Write string str to file.

        Note that due to buffering, flush() or close() are needed before
        the database reflects the data written.
        """
        _complain_ifclosed(self.closed)
        _complain_ifreadonly(self._isreadonly())
        return self._file.write(str)

    def writelines(self, iterable):
        """
        writelines(iterable) -> None.  Write the iterable of strings to file.

        Note that newlines are not added.  This is equalivalent to calling
        write() for each string in the iterable.
        """
        _complain_ifclosed(self.closed)
        _complain_ifreadonly(self._isreadonly())
        return self._file.writelines(iterable)

    def flush(self):
        """flush() -> None.  Flush the internal I/O buffer."""
        _complain_ifclosed(self.closed)
        _complain_ifreadonly(self._isreadonly())
        self.save()

    def save_base(self, *args, **kwargs):
        self.data = self._file.getvalue()
        self.size = self._size()
        super(File, self).save_base(*args, **kwargs)

    def _size(self):
        """_size() -> current file size, an integer (may be a long integer)."""
        position = self._file.tell()
        try:
            self._file.seek(0, 2)
            return self._file.tell()
        finally:
            self._file.seek(position)

    def _isreadonly(self):
        """_isreadonly() -> true or false.  True if mode is readonly."""
        return not ('+' in self.mode or 'w' in self.mode or 'a' in self.mode)


def _complain_ifreadonly(readonly):
    if readonly:
        raise IOError(EBADF, os.strerror(EBADF))



from __future__ import with_statement

from contextlib import contextmanager
import errno
from StringIO import StringIO

from django.conf import settings
from django.test import TestCase
from django.template import loader
from django.utils.functional import LazyObject

from django_dbstorage import views
from django_dbstorage.models import File
from django_dbstorage.storage import (DatabaseStorage,
                                      NonUniquifyingDatabaseStorage)


class FileTestCase(TestCase):
    def setUp(self):
        File.objects.all().delete()

    def test_init(self):
        f = File(name='hello.txt')
        self.assertEqual(f.name, 'hello.txt')
        self.assertEqual(f.data, '')
        self.assertEqual(f.closed, False)
        self.assertEqual(f.encoding, None)
        self.assertEqual(f.errors, None)
        self.assertEqual(f.mode, 'r+b')
        self.assertEqual(f.newlines, None)
        self.assertEqual(f.softspace, 0)

    def test_create(self):
        f = File.objects.create(name='hello.txt')
        self.assertEqual(f.name, 'hello.txt')
        self.assertEqual(f.data, '')
        self.assertEqual(f.closed, False)
        self.assertEqual(f.encoding, None)
        self.assertEqual(f.errors, None)
        self.assertEqual(f.mode, 'r+b')
        self.assertEqual(f.newlines, None)
        self.assertEqual(f.softspace, 0)

    def test_get(self):
        File.objects.create(name='hello.txt', data='Hello world!')
        f = File.objects.get(name='hello.txt')
        self.assertEqual(f.data, 'Hello world!')

    def test_unicode(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        self.assertEqual(repr(f), "<File: 'hello.txt', mode 'r+b'>")
        f.close()
        self.assertEqual(repr(f), "<File: closed 'hello.txt', mode 'r+b'>")

    def test_iter_next(self):
        data = ['Hello world!\n', 'Salut monde!\n']
        f = File.objects.create(name='hello.txt', data=''.join(data))
        for line, expected in zip(f, data):
            self.assertEqual(line, expected)

    def test_open(self):
        f = File.objects.create(name='hello.txt')
        self.assertFalse(f.closed)
        self.assertEqual(f.mode, 'r+b')
        # Reopen in the file in read-only mode
        f.open(mode='r')
        self.assertEqual(f.mode, 'r')
        # Reopen the file after moving the cursor
        f.seek(15)
        self.assertEqual(f.tell(), 15)
        f.open()
        self.assertEqual(f.mode, 'r')
        self.assertEqual(f.tell(), 0)
        # Reopen the file after closing
        f.close()
        self.assertTrue(f.closed)
        f.open()
        self.assertFalse(f.closed)
        # Attempt to reopen the file after seriously closing
        f.close()
        f._file = None
        self.assertRaises(ValueError, f.open)

    def test_close(self):
        f = File.objects.create(name='hello.txt')
        self.assertFalse(f.closed)
        f.close()
        self.assertTrue(f.closed)
        self.assertRaises(ValueError, f.isatty)
        self.assertRaises(ValueError, f.seek, 0)
        self.assertRaises(ValueError, f.tell)
        self.assertRaises(ValueError, f.read)
        self.assertRaises(ValueError, f.readline)
        self.assertRaises(ValueError, f.readlines)
        self.assertRaises(ValueError, f.truncate)
        self.assertRaises(ValueError, f.write, '')
        self.assertRaises(ValueError, f.writelines, [''])
        self.assertRaises(ValueError, f.flush)

    def test_isatty(self):
        f = File.objects.create(name='hello.txt')
        self.assertFalse(f.isatty())

    def test_seek_tell(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        self.assertEqual(f.tell(), 0)
        f.seek(5)
        self.assertEqual(f.tell(), 5)
        f.seek(3, 1)
        self.assertEqual(f.tell(), 8)
        f.seek(0, 2)
        self.assertEqual(f.tell(), 12)
        f.seek(-2, 2)
        self.assertEqual(f.tell(), 10)

    def test_read(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        self.assertEqual(f.read(1), 'H')
        self.assertEqual(f.read(), 'ello world!')

    def test_readline(self):
        f = File.objects.create(name='hello.txt',
                                data='Hello world!\nSalut monde!\n')
        self.assertEqual(f.readline(), 'Hello world!\n')
        self.assertEqual(f.readline(), 'Salut monde!\n')

    def test_readlines(self):
        f = File.objects.create(name='hello.txt',
                                data='Hello world!\nSalut monde!\n')
        self.assertEqual(f.readlines(), ['Hello world!\n',
                                         'Salut monde!\n'])

    def test_truncate(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        self.assertEqual(f.read(5), 'Hello')
        f.truncate()
        self.assertEqual(f.read(), '')
        f.seek(0)
        self.assertEqual(f.read(), 'Hello')
        f.seek(10)
        f.truncate(1)
        self.assertEqual(f.tell(), 10)
        f.seek(0)
        self.assertEqual(f.read(), 'H')

    def test_write(self):
        f = File.objects.create(name='hello.txt')
        f.write('Hello ')
        f.write('world!\n')
        print >>f, 'Salut monde!'
        f.flush()
        self.assertEqual(f.data, 'Hello world!\nSalut monde!\n')

    def test_writelines(self):
        f = File.objects.create(name='hello.txt')
        f.writelines(['Hello world!\n', 'Salut monde!\n'])
        f.flush()
        self.assertEqual(f.data, 'Hello world!\nSalut monde!\n')

    def test_flush(self):
        f = File.objects.create(name='hello.txt',
                                data='Hello world!\n')
        f.seek(0, 2)
        print >>f, 'Salut monde!'
        f.flush()
        f = File.objects.get(name='hello.txt')
        self.assertEqual(f.read(), 'Hello world!\nSalut monde!\n')

    def test_size(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        self.assertEqual(f._size(), 12)
        f.seek(5)
        f.truncate()
        self.assertEqual(f._size(), 5)

    def test_readonly(self):
        f = File.objects.create(name='hello.txt', data='Hello world!')
        f.mode = 'rb'
        self.assertFalse(f.isatty())
        self.assertEqual(f.tell(), 0)
        self.assertEqual(f.read(), 'Hello world!')
        f.seek(0)
        self.assertEqual(f.readline(), 'Hello world!')
        f.seek(0)
        self.assertEqual(f.readlines(), ['Hello world!'])
        self.assertRaises(IOError, f.truncate)
        self.assertRaises(IOError, f.write, '')
        self.assertRaises(IOError, f.writelines, [''])
        self.assertRaises(IOError, f.flush)


class DatabaseStorageTestCase(TestCase):
    def setUp(self):
        File.objects.all().delete()

    def test_init(self):
        s = DatabaseStorage()
        self.assertEqual(s.base_url, settings.MEDIA_URL)
        s = DatabaseStorage(base_url='/media/')
        self.assertEqual(s.base_url, '/media/')
        s = DatabaseStorage(base_url=None)
        self.assertEqual(s.base_url, None)
        s = DatabaseStorage()
        self.assertEqual(s.location, '')
        s = DatabaseStorage(location='media/')
        self.assertEqual(s.location, 'media/')

    def test_open(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        self.assertRaises(OSError, s.open, name='hello.txt')
        s.save(name='hello.txt', content=content)
        f = s.open(name='hello.txt')
        self.assertEqual(f.read(), content.getvalue())
        self.assertRaises(IOError, f.truncate)
        f = s.open(name='hello.txt', mode='r+b')
        self.assertEqual(f.read(), content.getvalue())
        f.truncate()

    def test_save(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        self.assertRaises(OSError, s.save, name='', content=content)
        self.assertRaises(OSError, s.save, name='/', content=content)
        self.assertRaises(OSError, s.save, name='hello/', content=content)
        self.assertEqual(s.save(name='hello.txt', content=content),
                         'hello.txt')
        self.assertEqual(s.save(name='hello.txt', content=content),
                         'hello_1.txt')
        self.assertEqual(s.save(name='/hello.txt', content=content),
                         'hello_2.txt')
        # Do not allow the storage layer to retry, if not unique.
        self.assertEqual(s._save(name='hello.txt', content=content),
                         'hello_3.txt')
        self.assertEqual(s.save(name='/hello/goodbye.txt', content=content),
                         'hello/goodbye.txt')
        # Try to save hello.txt, but without uniquify_names
        s = DatabaseStorage(uniquify_names=False)
        self.assertRaises(OSError, s._save, name='hello.txt', content=content)
        try:
            s._save(name='hello.txt', content=content)
        except OSError, e:
            self.assertEqual(e.errno, errno.EEXIST)

    def test_get_available_name(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        self.assertEqual(s.get_available_name('hello.txt'), 'hello.txt')
        s.save(name='hello.txt', content=content)
        self.assertEqual(s.get_available_name('hello.txt'), 'hello_1.txt')

    def test_delete(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        s.delete('hello.txt')
        s.save(name='hello.txt', content=content)
        s.delete('hello.txt')
        self.assertFalse(s.exists('hello.txt'))

    def test_exists(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        self.assertFalse(s.exists('hello.txt'))
        s.save(name='hello.txt', content=content)
        self.assertTrue(s.exists('hello.txt'))

    def test_listdir(self):
        content = StringIO('')
        english = StringIO('Hello world!')
        french = StringIO('Salut monde!')
        s = DatabaseStorage()
        self.assertEqual(s.listdir(''), ([], []))
        self.assertEqual(s.listdir('/'), ([], []))
        s.save(name='hello.txt', content=english)
        s.save(name='salut.txt', content=french)
        self.assertEqual(s.listdir(''), ([], ['hello.txt', 'salut.txt']))
        self.assertEqual(s.listdir('/'), ([], ['hello.txt', 'salut.txt']))
        s.save(name='hello/en.txt', content=english)
        s.save(name='hello/fr.txt', content=french)
        s.save(name='hello/docs/README', content=content)
        self.assertEqual(s.listdir(''), (['hello'],
                                         ['hello.txt', 'salut.txt']))
        self.assertEqual(s.listdir('/'), (['hello'],
                                          ['hello.txt', 'salut.txt']))
        self.assertEqual(s.listdir('hello'), (['docs'], ['en.txt', 'fr.txt']))
        self.assertEqual(s.listdir('/hello'), (['docs'], ['en.txt', 'fr.txt']))
        self.assertEqual(s.listdir('hello/docs'), ([], ['README']))
        self.assertEqual(s.listdir('/hello/docs'), ([], ['README']))
        self.assertRaises(OSError, s.listdir, 'goodbye')
        self.assertRaises(OSError, s.listdir, '/goodbye')

    def test_name(self):
        s = DatabaseStorage()
        self.assertEqual(s._name(''), '')
        self.assertEqual(s._name('.'), '')
        self.assertEqual(s._name('/'), '')
        self.assertEqual(s._name('//'), '')
        self.assertEqual(s._name('hello'), 'hello')
        self.assertEqual(s._name('/hello'), 'hello')
        self.assertEqual(s._name('//hello'), 'hello')
        self.assertEqual(s._name('hello/'), 'hello/')
        self.assertEqual(s._name('hello//'), 'hello/')
        self.assertEqual(s._name('hello/goodbye'), 'hello/goodbye')
        self.assertEqual(s._name('hello//goodbye'), 'hello/goodbye')

    def test_name_location(self):
        s = DatabaseStorage(location='root')
        self.assertEqual(s._name(''), 'root/')
        self.assertEqual(s._name('.'), 'root/')
        self.assertEqual(s._name('hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('/hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('hello/'), 'root/hello/')
        s = DatabaseStorage(location='root/')
        self.assertEqual(s._name(''), 'root/')
        self.assertEqual(s._name('.'), 'root/')
        self.assertEqual(s._name('hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('/hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('hello/'), 'root/hello/')
        s = DatabaseStorage(location='/root/')
        self.assertEqual(s._name(''), 'root/')
        self.assertEqual(s._name('.'), 'root/')
        self.assertEqual(s._name('hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('/hello.txt'), 'root/hello.txt')
        self.assertEqual(s._name('hello/'), 'root/hello/')

    def test_size(self):
        content = StringIO('Hello world!')
        s = DatabaseStorage()
        self.assertRaises(OSError, s.size, name='hello.txt')
        s.save(name='hello.txt', content=content)
        self.assertEqual(s.size(name='hello.txt'), len(content.getvalue()))

    def test_url(self):
        s = DatabaseStorage()
        s.base_url = None
        self.assertRaises(ValueError, s.url, name='hello.txt')
        s = DatabaseStorage(base_url='/media/')
        self.assertEqual(s.url(name='hello.txt'), '/media/hello.txt')
        self.assertEqual(s.url(name='/hello.txt'), '/hello.txt')

    def test_set_location(self):
        s = DatabaseStorage()
        self.assertEqual(s.location, '')
        s.set_location(None)
        self.assertEqual(s.location, '')
        s.set_location('')
        self.assertEqual(s.location, '')
        s.set_location('/')
        self.assertEqual(s.location, '')
        s.set_location('root')
        self.assertEqual(s.location, 'root/')
        s.set_location('root/')
        self.assertEqual(s.location, 'root/')
        s.set_location('root//')
        self.assertEqual(s.location, 'root/')
        s.set_location('/root/')
        self.assertEqual(s.location, 'root/')


class NonUniquifyingDatabaseStorageTestCase(TestCase):
    def setUp(self):
        File.objects.all().delete()

    def test_save(self):
        content = StringIO('Hello world!')
        s = NonUniquifyingDatabaseStorage()
        self.assertEqual(s._save(name='hello.txt', content=content),
                         'hello.txt')
        self.assertRaises(OSError, s._save, name='hello.txt', content=content)
        try:
            s._save(name='hello.txt', content=content)
        except OSError, e:
            self.assertEqual(e.errno, errno.EEXIST)


class ViewsTestCase(TestCase):
    urls = 'django_dbstorage.test_urls'

    def create(self, filename, content):
        f = StringIO(content)
        self.storage = DatabaseStorage()
        self.storage.save(name=filename, content=f)

    def assertRedirect(self, response, location):
        self.assertEqual(response.status_code, 301, response.content)
        self.assertEqual(response['Location'], location)

    def test_is_dir(self):
        self.assertTrue(views.is_dir(''))
        self.assertTrue(views.is_dir('.'))
        self.assertTrue(views.is_dir('..'))
        self.assertTrue(views.is_dir('/'))
        self.assertTrue(views.is_dir('dir/'))
        self.assertFalse(views.is_dir('file'))

    def _validate_path(self, path):
        class request:
            pass
        request.path = '/root/' + path
        return views.validate_path(request, path)

    def test_validate_path(self):
        self.assertEqual(self._validate_path(''), '')
        self.assertEqual(self._validate_path('hello%20world'), 'hello world')
        self.assertRedirect(self._validate_path('./hello%20world'),
                            '/root/hello%20world')
        self.assertRedirect(self._validate_path('.'), '/root/')
        self.assertRedirect(self._validate_path('/'), '/root/')
        self.assertRedirect(self._validate_path('./'), '/root/')
        self.assertEqual(self._validate_path('dir/'), 'dir/')
        self.assertEqual(self._validate_path('file'), 'file')
        self.assertRedirect(self._validate_path('dir/.'), '/root/dir/')
        self.assertRedirect(self._validate_path('dir/../DIR/'), '/root/DIR/')
        self.assertRedirect(self._validate_path('dir/../dir/..'), '/root/')
        self.assertRedirect(self._validate_path('../dir/'), '/dir/')
        self.assertRedirect(self._validate_path('../../dir/'), '/dir/')

    def test_urls(self):
        self.create('hello.txt', 'Hello')
        response = self.client.get('/urls/hello.txt')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(response.content, 'Hello')
        self.assertEqual(response['Content-Type'], 'text/plain')

    def test_urls_redirect(self):
        self.create('hello.txt', 'Hello')
        response = self.client.get('/urls/dir/../hello.txt')
        self.assertEqual(response.status_code, 301, response.content)
        self.assertEqual(response['Location'],
                         'http://testserver/urls/hello.txt')

    def test_urls_404(self):
        response = self.client.get('/urls/404')
        self.assertEqual(response.status_code, 404, response.content)

    def test_urls_index(self):
        response = self.client.get('/urls/')
        self.assertEqual(response.status_code, 404, response.content)

    def test_serve(self):
        self.create('hello.txt', 'Hello')
        response = self.client.get('/serve/hello.txt')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(response.content, 'Hello')
        self.assertEqual(response['Content-Type'], 'text/plain')

    def test_serve_404(self):
        response = self.client.get('/serve/404')
        self.assertEqual(response.status_code, 404, response.content)

    def test_serve_index(self):
        response = self.client.get('/serve/')
        self.assertEqual(response.status_code, 404, response.content)

    def test_indexes(self):
        self.create('hello.txt', 'Hello')
        with disable_templates():
            response = self.client.get('/indexes/')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertContains(response, 'hello.txt')

    def test_indexes_subdir(self):
        self.create('hello/world.txt', 'Hello world')
        with disable_templates():
            response = self.client.get('/indexes/')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertContains(response, 'hello')
        response = self.client.get('/indexes/hello')
        self.assertEqual(response.status_code, 404, response.content)
        with disable_templates():
            response = self.client.get('/indexes/hello/')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertContains(response, 'world.txt')

    def test_indexes_404(self):
        response = self.client.get('/indexes/hello/')
        self.assertEqual(response.status_code, 404, response.content)

    def test_root_exists(self):
        self.create('root/hello.txt', 'Hello')
        response = self.client.get('/root/hello.txt')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(response.content, 'Hello')
        self.assertEqual(response['Content-Type'], 'text/plain')

    def test_root_404(self):
        self.create('hello.txt', 'Hello')
        response = self.client.get('/root/hello.txt')
        self.assertEqual(response.status_code, 404, response.content)

    def test_root_indexes(self):
        self.create('root/hello.txt', 'Hello')
        with disable_templates():
            response = self.client.get('/root/')
        self.assertEqual(response.status_code, 200, response.content)
        self.assertContains(response, 'hello.txt')


@contextmanager
def disable_templates():
    """Disables TEMPLATE_LOADERS and user-defined templates."""
    with patch(loader, template_source_loaders=None):
        with patch(settings, TEMPLATE_LOADERS=[]):
            yield


@contextmanager
def patch(namespace, **values):
    """Patches `namespace`.`name` with `value` for (name, value) in values"""
    originals = {}
    if isinstance(namespace, LazyObject):
        if namespace._wrapped is None:
            namespace._setup()
        namespace = namespace._wrapped
    for (name, value) in values.iteritems():
        try:
            originals[name] = getattr(namespace, name)
        except AttributeError:
            originals[name] = NotImplemented
        if value is NotImplemented:
            if originals[name] is not NotImplemented:
                delattr(namespace, name)
        else:
            setattr(namespace, name, value)
    try:
        yield
    finally:
        for (name, original_value) in originals.iteritems():
            if original_value is NotImplemented:
                if values[name] is not NotImplemented:
                    delattr(namespace, name)
            else:
                setattr(namespace, name, original_value)



from errno import EEXIST, EISDIR, ENOENT, ENOTDIR
import itertools
import os
import posixpath
try:
    set
except NameError:
    from sets import Set as set
import urlparse

from django.conf import settings
from django.core.files.storage import Storage
from django.db import IntegrityError

from django_dbstorage.models import File


class DatabaseStorage(Storage):
    def __init__(self, location=None, base_url=settings.MEDIA_URL,
                 uniquify_names=True):
        self.set_location(location)
        self.base_url = base_url
        self.uniquify_names = uniquify_names

    def _open(self, name, mode='rb'):
        name = self._name(name)
        try:
            return File.objects.open(name=name, mode=mode)
        except File.DoesNotExist:
            raise OSError(ENOENT, os.strerror(ENOENT))

    def _save(self, name, content):
        name = self._name(name)
        if name.endswith(posixpath.sep):
            raise OSError(EISDIR, os.strerror(EISDIR))
        if not name:
            raise OSError(ENOENT, os.strerror(ENOENT))
        # Extract the data from content
        data = content.read()
        # Save to the database.
        while True:
            try:
                File.objects.create(name=posixpath.normpath(name), data=data)
            except IntegrityError:
                # File exists. We need a new file name.
                if not self.uniquify_names:
                    raise OSError(EEXIST, os.strerror(EEXIST))
                name = self.get_available_name(name)
            else:
                # OK, the file save worked. Break out of the loop.
                break
        return name

    def get_available_name(self, name):
        """
        Returns a filename that's free on the target storage system, and
        available for new content to be written to.
        """
        name = self._name(name)
        dir_name, file_name = posixpath.split(name)
        file_root, file_ext = os.path.splitext(file_name)
        # If the filename already exists, add an underscore and a number (before
        # the file extension, if one exists) to the filename until the generated
        # filename doesn't exist.
        count = itertools.count(1)
        while self.exists(name):
            # file_ext includes the dot.
            name = os.path.join(dir_name, "%s_%s%s" %
                                (file_root, count.next(), file_ext))
        return name

    def delete(self, name):
        """
        Deletes the specified file from the storage system.
        """
        name = self._name(name)
        File.objects.filter(name=name).delete()

    def exists(self, name):
        """
        Returns True if a file referened by the given name already exists in the
        storage system, or False if the name is available for a new file.
        """
        name = self._name(name)
        return bool(File.objects.filter(name=name).count())

    def listdir(self, path):
        """
        Lists the contents of the specified path, returning a 2-tuple of lists;
        the first item being directories, the second item being files.
        """
        path = self._name(path)
        if path and not path.endswith(posixpath.sep):
            path += posixpath.sep
        directories, files = set(), []
        entries = File.objects.filter(name__startswith=path)
        entries = entries.values_list('name', flat=True)
        if not entries and path:
            # Pretend empty directories don't exist, except for the root.
            raise OSError(ENOTDIR, os.strerror(ENOTDIR))
        for entry in entries:
            entry = entry[len(path):]
            bits = entry.split(posixpath.sep)
            if len(bits) == 1:
                files.append(bits[0])
            else:
                directories.add(bits[0])
        # Sort the directories and files
        directories = list(directories)
        directories.sort()
        files.sort()
        return directories, files

    def _name(self, name):
        name = name.lstrip(os.path.sep)
        new_name = os.path.normpath(name)
        if new_name == os.path.curdir:
            new_name = ''
        if self.location:
            new_name = os.path.join(self.location, new_name)
        if name.endswith(os.path.sep):
            # Preserve trailing slash for directories.
            new_name += os.path.sep
        # Convert os.path.sep into posixpath.sep
        new_name.replace(os.path.sep, posixpath.sep)
        return new_name

    def size(self, name):
        """
        Returns the total size, in bytes, of the file specified by name.
        """
        return self._open(name).size

    def url(self, name):
        """
        Returns an absolute URL where the file's contents can be accessed
        directly by a web browser.
        """
        if self.base_url is None:
            raise ValueError("This file is not accessible via a URL.")
        return urlparse.urljoin(self.base_url, name).replace('\\', '/')

    def set_location(self, location):
        if location:
            location = os.path.normpath(location).strip(os.path.sep)
            location = location.replace(os.path.sep, posixpath.sep)
        if not location:
            location = ''
        else:
            location += posixpath.sep
        self.location = location


class NonUniquifyingDatabaseStorage(DatabaseStorage):
    def __init__(self, location=None, base_url=None):
        return super(NonUniquifyingDatabaseStorage, self).__init__(
            location=location, base_url=base_url, uniquify_names=False
        )



# Copied from http://www.djangosnippets.org/snippets/1669/
#
# We would like to use a BlobField but that doesn't exist in Django yet:
# http://code.djangoproject.com/ticket/2417

import base64

from django.db import models


class Base64Field(models.TextField):

    def contribute_to_class(self, cls, name):
        if self.db_column is None:
            self.db_column = name
        self.field_name = name + '_base64'
        super(Base64Field, self).contribute_to_class(cls, self.field_name)
        setattr(cls, name, property(self.get_data, self.set_data))

    def get_data(self, obj):
        return base64.b64decode(getattr(obj, self.field_name))

    def set_data(self, obj, data):
        setattr(obj, self.field_name, base64.b64encode(data))



#!/usr/bin/env python
from setuptools import find_packages, setup

setup(
    name='django-dbstorage',
    version='1.3',
    description=('A Django file storage backend for files in the database.'),
    long_description=open('README.rst', 'r').read(),
    author='Akoha Inc.',
    author_email='adminmail@akoha.com',
    url='http://bitbucket.org/akoha/django-dbstorage/',
    packages=find_packages(),
    install_requires=['Django>=1.0'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    zip_safe=True,
)



"""
Test URLs for django_dbstorage.tests.ViewsTestCase.
"""

from django.conf.urls.defaults import include, patterns, url

from django_dbstorage.urls import serve


urlpatterns = patterns(
    '',
    url(r'^urls/', include('django_dbstorage.urls')),
    url(r'^serve/', include(serve())),
    url(r'^indexes/', include(serve(show_indexes=True))),
    url(r'^root/', include(serve(document_root='root', show_indexes=True))),
)



from itertools import chain
import mimetypes
import posixpath
import re
import urllib
import urlparse

from django.core.files.storage import get_storage_class
from django.http import Http404, HttpResponse, HttpResponsePermanentRedirect
from django.template import loader, Context, Template, TemplateDoesNotExist
from django.views.static import DEFAULT_DIRECTORY_INDEX_TEMPLATE


def is_dir(path):
    """Returns true if *path* refers to a directory."""
    head, part = posixpath.split(path)
    return part in ('', posixpath.curdir, posixpath.pardir)


def validate_path(request, path):
    """
    Clean up given *path* to only allow serving files below document_root.

    Returns a clean path, if there are no weird path elements.

    Otherwise, returns an HttpResponsePermanentRedirect to a clean path.

    Inspired by django.views.static.serve.
    """
    safepath = urllib.unquote(path)
    newpath = posixpath.normpath(safepath).lstrip(posixpath.sep)
    if newpath in (posixpath.curdir, posixpath.sep):
        newpath = ''
    elif newpath and is_dir(path):
        # Fix up ``newpath`` by appending ``/`` at the end, to make
        # it look like a directory.
        newpath += posixpath.sep
    # Redirect if there are weird *path* elements that were collapsed
    # out, or if ``newpath`` is attempted to go further up the tree.
    if newpath != safepath or \
       posixpath.pardir in newpath.split(posixpath.sep):
        # Figure out what the root URL is, by stripping out *path*
        # from the end of ``request.path``.
        root = re.sub(pattern=(r'(.*)' + re.escape(path)), repl=r'\1',
                      string=request.path)
        redirect = urlparse.urljoin(root, newpath)
        # Strip out any extraneous ``/..`` elements in the front of
        # ``redirect``, because you can't go too far up the host tree.
        newpath = re.sub(pattern=('^(?:%s%s)+' %
                                  (re.escape(posixpath.sep),
                                   re.escape(posixpath.pardir))),
                         repl=r'', string=redirect)
        return HttpResponsePermanentRedirect(newpath)
    return newpath


def serve(request, path, document_root=None, show_indexes=False,
          storage_class=None):
    """
    Serve static files below a given point in the directory structure.

    To use, put a URL pattern such as::

        (r'^(?P<path>.*)$',
         'django_dbstorage.views.serve',
         {'document_root' : '/path/to/my/files/'})

    in your URLconf. You must provide the ``document_root`` param. You may
    also set ``show_indexes`` to ``True`` if you'd like to serve a basic index
    of the directory.  This index view will use the template hardcoded below,
    but if you'd like to override it, you can create a template called
    ``static/directory_index.html``.

    Inspired by django.views.static.serve.
    """
    # Validate that the path is not malicious.
    path = validate_path(request, path)
    if isinstance(path, HttpResponse):
        # ``path`` is actually an HttpResponse and should be served.
        return path
    # Get the storage class, assuming that it's a DocumentStorage class.
    # If storage_class is None, we use settings.DEFAULT_FILE_STORAGE
    storage = get_storage_class(storage_class)(location=document_root)
    # Serve directory listings
    if is_dir(path):
        if show_indexes:
            try:
                return directory_index(storage, path)
            except OSError:
                pass
        raise Http404('Directory indexes are not allowed here.')
    # Serve the file directly out of DatabaseStorage
    try:
        contents = storage.open(path).read()
    except OSError:
        pass
    else:
        mimetype = (mimetypes.guess_type(path)[0] or
                    'application/octet-stream')
        response = HttpResponse(contents, mimetype=mimetype)
        response["Content-Length"] = len(contents)
        return response
    raise Http404('"%s" does not exist' % request.path)


def directory_index(storage, path):
    """
    Inspired by django.views.static.serve.
    """
    try:
        t = loader.select_template(['static/directory_index.html',
                                    'static/directory_index'])
    except TemplateDoesNotExist:
        t = Template(DEFAULT_DIRECTORY_INDEX_TEMPLATE,
                     name='Default directory index template')
    directories, files = storage.listdir(path)
    file_list = chain((f + '/' for f in directories if not f.startswith('.')),
                      (f for f in files if not f.startswith('.')))
    c = Context({
        'directory': path.rstrip('/') + '/',
        'file_list': file_list,
    })
    return HttpResponse(t.render(c))



"""
URL patterns to serve files from ``django_dbstorage``.

You can either use it as a module::

    url(r'^media/', include('django_dbstorage.urls'))

Or you can customize how the files are served::

    url(r'^media/',
        include(django_dbstorage.urls.serve(document_root='/root',
                                            show_indexes=True,
                                            name='serve')))
"""
from django.conf.urls.defaults import patterns, url


def serve(document_root=None, show_indexes=False,
          storage_class='django_dbstorage.storage.DatabaseStorage',
          name='dbstorage_serve'):
    """
    Returns URL patterns for the DatabaseStorage with several options.

    *document_root*
        See ``django_dbstorage.views.serve``.

    *show_indexes*
        See ``django_dbstorage.views.serve``.

    *storage_class*
        Specify a different DatabaseStorage class to use for serving
        files. This is useful if it has been subclassed.

    *name*
        Name the URL. Useful if serving from multiple URLs.
    """
    return patterns(
        '',
        url(r'^(?P<path>.*)$', 'django_dbstorage.views.serve',
            name=name,
            kwargs={'document_root': document_root,
                    'show_indexes': show_indexes,
                    'storage_class': storage_class}),
    )


#: Default ``urlpatterns`` so you can ``include('django_dbstorage.urls')``
urlpatterns = serve()
